# ##### BEGIN GPL LICENSE BLOCK #####
#
#  This program is free software; you can redistribute it and/or
#  modify it under the terms of the GNU General Public License
#  as published by the Free Software Foundation; either version 2
#  of the License, or (at your option) any later version.
#
#  This program is distributed in the hope that it will be useful,
#  but WITHOUT ANY WARRANTY; without even the implied warranty of
#  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#  GNU General Public License for more details.
#
#  You should have received a copy of the GNU General Public License
#  along with this program; if not, write to the Free Software Foundation,
#  Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
#
# ##### END GPL LICENSE BLOCK #####

bl_info = {
    "name": "Encode HDR",
    "author": "Fredrik Hansson",
    "version": (1,0),
    "blender": (2, 6, 1),
    "location": "UV/Image editor> Image > Encode HDR",
    "description": "Encodes HDR to a format as a workaround for the problem that cycles only supports 8bit/channel",
    "warning": "", # used for warning icon and text in addons panel
    "wiki_url": '',
    "tracker_url": "" ,
    "category": "Image"}

import bpy
import os
import math
from math import *

from array import array
 
# should really be one single image but cycles does not support texture alpha output from texture nodes atm
def go(target,target2,length):
    
    for index in range(0,length):
        r = target[index*4]
        g = target[index*4+1]
        b = target[index*4+2]
        maxcomponent=max(r,max(g,b))
        if maxcomponent<0.000001:
            maxcomponent=0.000001
        exponent=ceil(log(maxcomponent,2))
        pow2=pow(2,exponent)
        target[index*4] =r/pow2;
        target[index*4+1]=g/pow2
        target[index*4+2]=b/pow2;
        target2[index*4]=(exponent+128.0)/255.0
        target2[index*4+1]=(exponent+128.0)/255.0
        target2[index*4+2]=(exponent+128.0)/255.0

def Encode(context):
    HDRimage = context.space_data.image
    outputnameRGB="{0}_{1}.png".format(os.path.splitext(HDRimage.name)[0],'EncodedRGB')
    outputnameEXP="{0}_{1}.png".format(os.path.splitext(HDRimage.name)[0],'EncodedEXP')
    #print(HDRimage.name)
    #print(outputnameRGB)
    #print(outputnameEXP)
    
    bpy.ops.image.new(name=outputnameRGB,width=HDRimage.size[0],height=HDRimage.size[1])    
    outputimageRGB=bpy.data.images[outputnameRGB]
    bpy.ops.image.new(name=outputnameEXP,width=HDRimage.size[0],height=HDRimage.size[1])
    outputimageEXP=bpy.data.images[outputnameEXP]
    outputimageRGB.file_format='PNG'
    outputimageEXP.file_format='PNG'
    
    pixels = array('f',HDRimage.pixels)
    pixels2 = array('f',HDRimage.pixels)
    go(pixels,pixels2,int(len(pixels)/4))
    outputimageRGB.pixels=pixels
    outputimageRGB.update()
    outputimageEXP.pixels=pixels2
    outputimageEXP.update()
    
    outputimageRGB.save_render(os.path.splitext(HDRimage.filepath)[0]+'_EncodedRGB.png')
    outputimageEXP.save_render(os.path.splitext(HDRimage.filepath)[0]+'_EncodedEXP.png')
    outputimageRGB.filepath=os.path.splitext(HDRimage.filepath)[0]+'_EncodedRGB.png'
    outputimageEXP.filepath=os.path.splitext(HDRimage.filepath)[0]+'_EncodedEXP.png'
    outputimageRGB.source='FILE'
    outputimageEXP.source='FILE'
    return outputimageRGB,outputimageEXP

    
def CreateNodeGroup(context,rgbimage,expimage):
    nodegroup=bpy.data.worlds[0].node_tree
    nodegroup.nodes.clear()
    
    multiply1=nodegroup.nodes.new('MATH')
    multiply1.name='Multiply1'
    multiply1.operation='MULTIPLY'
    
    multiply2=nodegroup.nodes.new('MIX_RGB')
    multiply2.name='Multiply2'
    multiply2.blend_type='MULTIPLY'
    multiply2.inputs[0].default_value=1.0
    
    subtract1=nodegroup.nodes.new('MATH')
    subtract1.name='Subtract'
    subtract1.inputs[1].default_value=0.5
    subtract1.operation='SUBTRACT'
    
    power1=nodegroup.nodes.new('MATH')
    power1.name='Power'
    power1.inputs[0].default_value=2.0
    power1.operation='POWER'
    
    value1=nodegroup.nodes.new('VALUE')
    value1.name='Value'
    value1.outputs[0].default_value=255.0
    
    gamma1=nodegroup.nodes.new('GAMMA')
    gamma1.name='Gamma'
    gamma1.inputs[1].default_value=2.2
    
    separatergb1=nodegroup.nodes.new('SEPRGB')
    separatergb1.name='Separate RGB'
    
    GammaInput=nodegroup.inputs.new(type='VALUE',name='Gamma')
    GammaInput.default_value=2.2
    
    Output=nodegroup.nodes.new('OUTPUT_WORLD')
    Background=nodegroup.nodes.new('BACKGROUND')
    
    TexRGB=nodegroup.nodes.new('TEX_ENVIRONMENT')
    TexRGB.color_space='LINEAR'
    TexRGB.image=rgbimage
    TexEXP=nodegroup.nodes.new('TEX_ENVIRONMENT')
    TexEXP.color_space='LINEAR'
    TexEXP.image=expimage
    
    Mapping=nodegroup.nodes.new('MAPPING')
    TexCoord=nodegroup.nodes.new('TEX_COORD')
    
    
    TexCoord.location[0]=-250
    TexCoord.location[1]=-225
    TexRGB.location[0]=350
    TexRGB.location[1]=0
    TexEXP.location[0]=350
    TexEXP.location[1]=-160
    separatergb1.location[0]=550
    separatergb1.location[1]=-160
    subtract1.location[0]=700
    subtract1.location[1]=-160
    value1.location[0]=700
    value1.location[1]=-320
    multiply1.location[0]=900
    multiply1.location[1]=-160
    power1.location[0]=1100
    power1.location[1]=-160
    multiply2.location[0]=1300
    multiply2.location[1]=0
    gamma1.location[0]=1500
    gamma1.location[1]=0
    Background.location[0]=1700
    Background.location[1]=0
    Output.location[0]=1900
    Output.location[1]=0
    
    
    
    nodegroup.links.new(multiply2.inputs[1],TexRGB.outputs[0])
    nodegroup.links.new(separatergb1.inputs[0],TexEXP.outputs[0])
    
    nodegroup.links.new(Background.inputs[0],gamma1.outputs[0])
    nodegroup.links.new(Output.inputs[0],Background.outputs[0])
    nodegroup.links.new(gamma1.inputs[0],multiply2.outputs[0])
    nodegroup.links.new(power1.outputs[0],multiply2.inputs[2])
    nodegroup.links.new(separatergb1.outputs[0],subtract1.inputs[0])
    nodegroup.links.new(subtract1.outputs[0],multiply1.inputs[0])
    nodegroup.links.new(value1.outputs[0],multiply1.inputs[1])
    nodegroup.links.new(multiply1.outputs[0],power1.inputs[1])
    
    nodegroup.links.new(TexCoord.outputs[0],Mapping.inputs[0])
    nodegroup.links.new(Mapping.outputs[0],TexRGB.inputs[0])
    nodegroup.links.new(Mapping.outputs[0],TexEXP.inputs[0])
    
    

class EncodeHDR(bpy.types.Operator):
    ''''''
    bl_idname = "image.encodehdr"
    bl_label = "EncodeHDR"
    bl_options = {'REGISTER'}


    def execute(self, context):
        Encode(context)
        return {'FINISHED'}
        
class DecodeHDRGroup(bpy.types.Operator):
    ''''''
    bl_idname = "image.decodehdrgroups"
    bl_label = "Create DecodeHDR Node Group"
    bl_options = {'REGISTER'}


    def execute(self, context):
        CreateNodeGroup(context,None,None)
        return {'FINISHED'}
        
class UseAsHDR(bpy.types.Operator):
    ''''''
    bl_idname = "image.useashdr"
    bl_label = "Use As HDR environment"
    bl_options = {'REGISTER'}


    def execute(self, context):
        imagergb,imageexp=Encode(context)
        CreateNodeGroup(context,imagergb,imageexp)
        return {'FINISHED'}


def menu_func(self, context):
    self.layout.operator(EncodeHDR.bl_idname)
def menu_func_decode(self, context):
    self.layout.operator(DecodeHDRGroup.bl_idname)
def menu_func_useashdr(self, context):
    self.layout.operator(UseAsHDR.bl_idname)

def register():
    bpy.utils.register_module(__name__)

    bpy.types.IMAGE_MT_image.append(menu_func)
    bpy.types.IMAGE_MT_image.append(menu_func_decode)
    bpy.types.IMAGE_MT_image.append(menu_func_useashdr)

def unregister():
    bpy.utils.unregister_module(__name__)

    bpy.types.IMAGE_MT_image.remove(menu_func)
    bpy.types.IMAGE_MT_image.remove(menu_func_decode)
    bpy.types.IMAGE_MT_image.remove(menu_func_useashdr)

if __name__ == "__main__":
    register()
